#include "MGXSUtils/MatxsUtils/MatxsObject.hpp"

using namespace frendy;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

// ***** MatxsCommonData ***** //

//constructor
MatxsCommonData::MatxsCommonData(void)
{
  clear();
}

//destructor
MatxsCommonData::~MatxsCommonData(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::clear()
{
  //0v
  hname.clear();
  huse.clear();

  ivers = 0;

  //1d
  npart  = 0;
  ntype  = 0;
  nholl  = 0;
  nmat   = 0;
  matxw  = matxw_def;
  length = 0;

  //2d
  hsetid.clear();

  //3d
  hpart.clear();
  htype.clear();
  hmatn.clear();
  ngrp.clear();
  jinp.clear();
  joutp.clear();
  nsubm.clear();
  locm.clear();

  //4d
  vector<MatxsGroupStructureData>().swap(grp_obj);

  //5-10d
  vector<MatxsMaterialData>().swap(mat_obj);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::read_data(vector<string>& text_vec)
{
  int line_no = 0;
  read_data(text_vec, line_no);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::read_data(vector<string>& text_vec, int& line_no)
{
  search_0v_data(text_vec, line_no);

  StringUtils str_obj;
  string class_name = "MatxsCommonData";
  string func_name  = "read_data(vector<string>& text_vec, int& line_no)";
  str_obj.check_string_vector_data_end(text_vec, line_no+5, class_name, func_name);

  read_0v_data(text_vec, line_no);
  read_1d_data(text_vec, line_no);
  read_2d_data(text_vec, line_no);
  read_3d_data(text_vec, line_no);

  //Read 4d data
  grp_obj.resize(npart);
  for(int i=0; i<npart; i++)
  {
    grp_obj[i].set_ngrp(ngrp[i]);
    grp_obj[i].read_data(text_vec, line_no);
  }

  //Read 5d - 10d data
  mat_obj.resize(nmat);
  for(int i=0; i<nmat; i++)
  {
    mat_obj[i].set_ngrp(ngrp);
    mat_obj[i].set_joutp(joutp);
    mat_obj[i].set_nsubm(nsubm[i]);
    mat_obj[i].read_data(text_vec, line_no);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::search_0v_data(vector<string>& text_vec, int& line_no)
{
  if( line_no < 0 )
  {
    line_no = 0;
  }

  int find_flg = -1;
  for(int i=line_no; i<static_cast<int>(text_vec.size()); i++)
  {
    if( text_vec[i].find("0v") != string::npos )
    {
      find_flg = i;
      break;
    }
  }

  if( find_flg < 0 )
  {
    string class_name = "MatxsCommonData";
    string func_name  = "search_0v_data";

    vector<string> err_com;
    err_com.push_back("The 0v data is not found in the text data.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  line_no = find_flg;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::read_0v_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  vector<string> str_vec;

  if( static_cast<int>(text_vec[line_no].length()) <= 30 )
  {
    string class_name = "MatxsCommonData";
    string func_name  = "read_0v_data";

    string str_data = "line data : " + text_vec[line_no];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("The number of characters in 0v data must be larger than 31.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  str_vec.resize(6);
  str_vec[0] = text_vec[line_no].substr( 0, 4); // 0v
  str_vec[1] = text_vec[line_no].substr( 4, 8); //hname
  str_vec[2] = text_vec[line_no].substr(12, 1); // *
  str_vec[3] = text_vec[line_no].substr(13,16); //huse
  str_vec[4] = text_vec[line_no].substr(29, 1);
  str_vec[5] = text_vec[line_no].substr(30);
  str_vec[5] = str_obj.get_formatted_string_data(str_vec[5], 6);

  if( str_vec[0].find("0v") == string::npos )
  {
    string class_name = "MatxsCommonData";
    string func_name  = "read_0v_data";

    string str_data = "line data : " + text_vec[line_no];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 0v data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }
  line_no++;
  str_obj.skip_no_space_string_data(text_vec, line_no);

  hname = str_vec[1];
  huse  = str_vec[3];
  ivers = numeric_string_cast<Integer>(str_vec[5]);
  str_vec.clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::read_1d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  vector<string> str_vec;

  str_vec = str_obj.divide_string_data(text_vec[line_no], 6, 42);

  if( str_vec[0].find("1d") == string::npos )
  {
    string class_name = "MatxsCommonData";
    string func_name  = "read_1d_data";

    string str_data = "line data : " + text_vec[line_no];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 1d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }
  line_no++;
  str_obj.skip_no_space_string_data(text_vec, line_no);

  npart  = numeric_string_cast<Integer>(str_vec[1]);
  ntype  = numeric_string_cast<Integer>(str_vec[2]);
  nholl  = numeric_string_cast<Integer>(str_vec[3]);
  nmat   = numeric_string_cast<Integer>(str_vec[4]);
  matxw  = numeric_string_cast<Integer>(str_vec[5]);
  length = numeric_string_cast<Integer>(str_vec[6]);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::read_2d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsCommonData";
  string func_name  = "read_2d_data";

  if( text_vec[line_no].find("2d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 2d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }
  line_no++;
  str_obj.skip_no_space_string_data(text_vec, line_no);

  int line_no_hsetid = nholl / 9;
  hsetid.resize(static_cast<int>(line_no_hsetid));
  for(int i=0; i<static_cast<int>(line_no_hsetid); i++)
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    hsetid[i] = str_obj.get_formatted_string_data(text_vec[line_no], 72);
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::read_3d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsCommonData";
  string func_name  = "read_3d_data";
  vector<string> str_vec;

  int line_no_ori = line_no;
  int data_no     = npart + ntype + nmat + 1;
  while( static_cast<int>(str_vec.size()) < data_no )
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    str_obj.add_string_vector_data(str_vec, str_obj.divide_string_data(text_vec[line_no], 8, 72));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  if( str_vec[0].find("3d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no_ori];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 3d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  int ele_no = 1;
  hpart.resize(npart);
  htype.resize(ntype);
  hmatn.resize(nmat);
  for(int i=0; i<npart; i++)
  {
    hpart[i] = str_vec[ele_no];
    ele_no++;
  }
  for(int i=0; i<ntype; i++)
  {
    htype[i] = str_vec[ele_no];
    ele_no++;
  }
  for(int i=0; i<nmat; i++)
  {
    hmatn[i] = str_vec[ele_no];
    ele_no++;
  }
  str_vec.clear();

  data_no = npart + ntype*2 + nmat*2;
  while( static_cast<int>(str_vec.size()) < data_no )
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    str_obj.add_string_vector_data(str_vec, str_obj.divide_string_data(text_vec[line_no], 6, 72));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  ele_no = 0;
  ngrp.resize(npart);
  jinp.resize(ntype);
  joutp.resize(ntype);
  nsubm.resize(nmat);
  locm.resize(nmat);
  for(int i=0; i<npart; i++)
  {
    ngrp[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  for(int i=0; i<ntype; i++)
  {
    jinp[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  for(int i=0; i<ntype; i++)
  {
    joutp[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  for(int i=0; i<nmat; i++)
  {
    nsubm[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  for(int i=0; i<nmat; i++)
  {
    locm[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  str_vec.clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<string> MatxsCommonData::write_data()
{
  StringUtils str_obj;

  vector<string> text;
  text.clear();

  //0v
  string header = " 0v";

  vector<string> str_vec;
  str_vec.resize(6);
  str_vec[0] = str_obj.get_formatted_string_data(header, 4);
  str_vec[1] = str_obj.get_formatted_string_data(hname, 8);
  str_vec[2] = "*";
  str_vec[3] = str_obj.get_formatted_string_data(huse,16);
  str_vec[4] = "*";
  str_vec[5] = str_obj.get_formatted_int_data(ivers,-6);

  text.push_back(str_obj.get_merged_string_vector_data(str_vec));
  str_vec.clear();

  //1d
  header = " 1d";
  calc_1d_data();
  str_vec.resize(7);
  str_vec[0] = str_obj.get_formatted_string_data(header, 6);
  str_vec[1] = str_obj.get_formatted_int_data(npart, -6);
  str_vec[2] = str_obj.get_formatted_int_data(ntype, -6);
  str_vec[3] = str_obj.get_formatted_int_data(nholl, -6);
  str_vec[4] = str_obj.get_formatted_int_data(nmat,  -6);
  str_vec[5] = str_obj.get_formatted_int_data(matxw, -6);
  str_vec[6] = str_obj.get_formatted_int_data(length,-6);

  text.push_back(str_obj.get_merged_string_vector_data(str_vec));
  str_vec.clear();

  //2d
  header = " 2d";

  str_vec.resize(1);
  str_vec[0] = str_obj.get_formatted_string_data(header, 4);
  text.push_back(str_obj.get_merged_string_vector_data(str_vec));

  for(int i=0; i<static_cast<int>(hsetid.size()); i++)
  {
    str_vec[0] = str_obj.get_formatted_string_data(hsetid[i], 72);
    text.push_back(str_obj.get_merged_string_vector_data(str_vec));
  }
  str_vec.clear();

  //3d
  header = " 3d";

  str_vec.push_back(str_obj.get_formatted_string_data(header, 8));
  for(int i=0; i<npart; i++)
  {
    str_vec.push_back(str_obj.get_formatted_string_data(hpart[i], 8));
  }
  for(int i=0; i<ntype; i++)
  {
    str_vec.push_back(str_obj.get_formatted_string_data(htype[i], 8));
  }
  for(int i=0; i<nmat; i++)
  {
    str_vec.push_back(str_obj.get_formatted_string_data(hmatn[i], 8));
  }
  str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 9));
  str_vec.clear();

  for(int i=0; i<npart; i++)
  {
    str_vec.push_back(str_obj.get_formatted_int_data(ngrp[i],-6));
  }
  for(int i=0; i<ntype; i++)
  {
    str_vec.push_back(str_obj.get_formatted_int_data(jinp[i],-6));
  }
  for(int i=0; i<ntype; i++)
  {
    str_vec.push_back(str_obj.get_formatted_int_data(joutp[i],-6));
  }
  for(int i=0; i<nmat; i++)
  {
    str_vec.push_back(str_obj.get_formatted_int_data(nsubm[i],-6));
  }
  for(int i=0; i<nmat; i++)
  {
    str_vec.push_back(str_obj.get_formatted_int_data(locm[i],-6));
  }
  str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 12));
  str_vec.clear();

  //Write 4d data
  for(int i=0; i<npart; i++)
  {
    str_obj.add_string_vector_data(text, grp_obj[i].write_data());
  }

  //Write 5-10d data
  for(int i=0; i<nmat; i++)
  {
    str_obj.add_string_vector_data(text, mat_obj[i].write_data());
  }

  return text;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::mod_tsl_name(Integer mt_no)
{
  string tsl_name = get_tsl_name(mt_no);
  if( static_cast<int>(tsl_name.length()) == 0 )
  {
    //Do not modify tsl name
    return;
  }

  int i_max = static_cast<int>(mat_obj.size());
  for(int i=0; i<i_max; i++)
  {
    vector<MatxsSubmaterialData> submat_obj = mat_obj[i].get_submat_obj();

    int j_max = static_cast<int>(submat_obj.size());
    for(int j=0; j<j_max; j++)
    {
      //Modify 6d data
      vector<string> hvps = submat_obj[j].get_hvps();
      int k_max = static_cast<int>(hvps.size());
      for(int k=0; k<k_max; k++)
      {
        mod_tsl_name(hvps[k], mt_no);
      }
      submat_obj[j].set_hvps(hvps);
      hvps.clear();

      //Modify 8d data
      vector<MatxsMatrixData> matrix_obj = submat_obj[j].get_matrix_obj();
      k_max = static_cast<int>(matrix_obj.size());
      for(int k=0; k<k_max; k++)
      {
        string hmtx = matrix_obj[k].get_hmtx();
        mod_tsl_name(hmtx, mt_no);
        matrix_obj[k].set_hmtx(hmtx);
        hmtx.clear();
      }
      submat_obj[j].set_matrix_obj(matrix_obj);
      vector<MatxsMatrixData>().swap(matrix_obj);
    }

    mat_obj[i].set_submat_obj(submat_obj);
    vector<MatxsSubmaterialData>().swap(submat_obj);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::mod_tsl_name(string& str_data, Integer mt_no)
{
  vector<string> tsl_name_ori;
  tsl_name_ori.push_back("other");
  tsl_name_ori.push_back("free");
  string str_data_mod = str_data;

  StringUtils str_obj;
  str_data_mod = str_obj.trim_space(str_data_mod);
  transform(str_data_mod.begin(), str_data_mod.end(), str_data_mod.begin(), ::tolower);
  
  for(int i=0; i<static_cast<int>(tsl_name_ori.size()); i++)
  {
    if( str_data_mod == tsl_name_ori[i] )
    {
      str_data_mod = get_tsl_name(mt_no);
      if( static_cast<int>(str_data_mod.length()) > 0 )
      {
        str_data = str_data_mod;
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

string MatxsCommonData::get_tsl_name(Integer mt_no)
{
  int i_max = 26;
  vector<string> tsl_name_list;
  tsl_name_list.resize(i_max);

  tsl_name_list[ 0] = "free";  //221
  tsl_name_list[ 1] = "hh2o";  //222
  tsl_name_list[ 2] = "poly";  //223
  tsl_name_list[ 3] = "poly";  //224
  tsl_name_list[ 4] = "hzrh";  //225
  tsl_name_list[ 5] = "hzrh";  //226
  tsl_name_list[ 6] = "benz";  //227
  tsl_name_list[ 7] = "dd2o";  //228
  tsl_name_list[ 8] = "graph"; //229
  tsl_name_list[ 9] = "graph"; //230
  tsl_name_list[10] = "be";    //231
  tsl_name_list[11] = "be";    //232
  tsl_name_list[12] = "bebeo"; //233
  tsl_name_list[13] = "bebeo"; //234
  tsl_name_list[14] = "zrzrh"; //235
  tsl_name_list[15] = "zrzrh"; //236
  tsl_name_list[16] = "obeo";  //237
  tsl_name_list[17] = "obeo";  //238
  tsl_name_list[18] = "ouo2";  //239
  tsl_name_list[19] = "ouo2";  //240
  tsl_name_list[20] = "uuo2";  //241
  tsl_name_list[21] = "uuo2";  //242
  tsl_name_list[22] = "al";    //243
  tsl_name_list[23] = "al";    //244
  tsl_name_list[24] = "fe";    //245
  tsl_name_list[25] = "fe";    //246

  string tsl_name = "";
  tsl_name.clear();

  int pos = static_cast<int>(mt_no) - 221;
  if( pos >= 0 && pos < i_max )
  {
    tsl_name = tsl_name_list[pos];
  }
  else if( pos >= 0 && mt_no <= 250 )
  {
    tsl_name = "other";
  }

  return tsl_name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsCommonData::calc_1d_data()
{
  npart = static_cast<Integer>(grp_obj.size());
  ntype = static_cast<Integer>(htype.size());
  nholl = static_cast<Integer>(hsetid.size()) * 9;
  nmat  = static_cast<Integer>(mat_obj.size());
  matxw = matxw_def;

  length = 6;
  for(int i=0; i<npart; i++)
  {
    length += grp_obj[i].get_loc_no();
  }

  for(int i=0; i<nmat; i++)
  {
    length += mat_obj[i].get_loc_no();
  }


  for(int i=0; i<nmat; i++)
  {
    nsubm[i] = static_cast<Integer>(mat_obj[i].get_temp().size());
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Setter
//0v
void MatxsCommonData::set_hname(string str_data)
{
  hname = str_data;
}
void MatxsCommonData::set_huse(string str_data)
{
  huse = str_data;
}
void MatxsCommonData::set_ivers(Integer int_data)
{
  ivers = int_data;
}

//1d
void MatxsCommonData::set_npart(Integer int_data)
{
  npart = int_data;
}
void MatxsCommonData::set_ntype(Integer int_data)
{
  ntype = int_data;
}
void MatxsCommonData::set_nholl(Integer int_data)
{
  nholl = int_data;
}
void MatxsCommonData::set_nmat(Integer int_data)
{
  nmat = int_data;
}
void MatxsCommonData::set_matxw(Integer int_data)
{
  matxw = int_data;
}
void MatxsCommonData::set_length(Integer int_data)
{
  length = int_data;
}

//2d
void MatxsCommonData::set_hsetid(vector<string> str_vec)
{
  hsetid = str_vec;
}

//3d
void MatxsCommonData::set_hpart(vector<string> str_vec)
{
  hpart = str_vec;
}
void MatxsCommonData::set_htype(vector<string> str_vec)
{
  htype = str_vec;
}
void MatxsCommonData::set_hmatn(vector<string> str_vec)
{
  hmatn = str_vec;
}
void MatxsCommonData::set_ngrp(vector<Integer> int_vec)
{
  ngrp = int_vec;
}
void MatxsCommonData::set_jinp(vector<Integer> int_vec)
{
  jinp = int_vec;
}
void MatxsCommonData::set_joutp(vector<Integer> int_vec)
{
  joutp = int_vec;
}
void MatxsCommonData::set_nsubm(vector<Integer> int_vec)
{
  nsubm = int_vec;
}
void MatxsCommonData::set_locm(vector<Integer> int_vec)
{
  locm = int_vec;
}

//4d
void MatxsCommonData::set_grp_obj(vector<MatxsGroupStructureData> obj_vec)
{
  grp_obj = obj_vec;
  vector<MatxsGroupStructureData>().swap(obj_vec);
}

//5-10d
void MatxsCommonData::set_mat_obj(vector<MatxsMaterialData> obj_vec)
{
  mat_obj = obj_vec;
  vector<MatxsMaterialData>().swap(obj_vec);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Getter
//0v
string  MatxsCommonData::get_hname()
{
  return hname;
}
string  MatxsCommonData::get_huse()
{
  return huse;
}
Integer MatxsCommonData::get_ivers()
{
  return ivers;
}

//1d
Integer MatxsCommonData::get_npart()
{
  calc_1d_data();
  return npart;
}
Integer MatxsCommonData::get_ntype()
{
  calc_1d_data();
  return ntype;
}
Integer MatxsCommonData::get_nholl()
{
  calc_1d_data();
  return nholl;
}
Integer MatxsCommonData::get_nmat()
{
  calc_1d_data();
  return nmat;
}
Integer MatxsCommonData::get_matxw()
{
  calc_1d_data();
  return matxw;
}
Integer MatxsCommonData::get_length()
{
  calc_1d_data();
  return length;
}

//2d
vector<string> MatxsCommonData::get_hsetid()
{
  return hsetid;
}

//3d
vector<string>  MatxsCommonData::get_hpart()
{
  return hpart;
}
vector<string>  MatxsCommonData::get_htype()
{
  return htype;
}
vector<string>  MatxsCommonData::get_hmatn()
{
  return hmatn;
}
vector<Integer> MatxsCommonData::get_ngrp()
{
  return ngrp;
}
vector<Integer> MatxsCommonData::get_jinp()
{
  return jinp;
}
vector<Integer> MatxsCommonData::get_joutp()
{
  return joutp;
}
vector<Integer> MatxsCommonData::get_nsubm()
{
  return nsubm;
}
vector<Integer> MatxsCommonData::get_locm()
{
  return locm;
}

//4d
vector<MatxsGroupStructureData> MatxsCommonData::get_grp_obj()
{
  return grp_obj;
}

//5-10d
vector<MatxsMaterialData> MatxsCommonData::get_mat_obj()
{
  return mat_obj;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

// ***** MatxsGroupStructureData ***** //

//constructor
MatxsGroupStructureData::MatxsGroupStructureData(void)
{
  clear();
}

//destructor
MatxsGroupStructureData::~MatxsGroupStructureData(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsGroupStructureData::clear()
{
  ngrp = 0;

  //4d
  gpb.clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsGroupStructureData::read_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsGroupStructureData";
  string func_name  = "read_data";
  vector<string> str_vec;

  if( ngrp == 0 )
  {
    vector<string> err_com;
    err_com.push_back("Please set ngrp value to read 4d data.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  int line_no_ori = line_no;
  int data_no     = ngrp + 2;

  str_vec.clear();
  while( static_cast<int>(str_vec.size()) < data_no )
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    str_obj.add_string_vector_data(str_vec, str_obj.divide_string_data(text_vec[line_no], 12, 72));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  if( str_vec[0].find("4d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no_ori];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 4d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  gpb.resize(ngrp+1);
  for(int i=1; i<data_no; i++)
  {
    gpb[i-1] = numeric_string_cast<Real>(str_vec[i]);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<string> MatxsGroupStructureData::write_data()
{
  StringUtils str_obj;

  vector<string> text, str_vec;
  text.clear();
  str_vec.clear();

  string header = " 4d";
  str_vec.push_back(str_obj.get_formatted_string_data(header, 12));
  for(int i=0; i<static_cast<int>(gpb.size()); i++)
  {
    str_vec.push_back(str_obj.get_formatted_real_data(gpb[i], 12));
  }
  str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 6));
  str_vec.clear();

  return text; 
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Integer MatxsGroupStructureData::get_loc_no()
{
  Integer loc_no = 0;

  //Count 4d
  if( static_cast<int>(gpb.size()) > 0 )
  {
    loc_no++;
  }

  return loc_no;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Setter
void MatxsGroupStructureData::set_ngrp(int int_data)
{
  ngrp = int_data;
}

void MatxsGroupStructureData::set_gpb(vector<Real> real_vec)
{
  gpb = real_vec;
}

//Getter
int          MatxsGroupStructureData::get_ngrp()
{
  return ngrp;
}

vector<Real> MatxsGroupStructureData::get_gpb()
{
  return gpb;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

// ***** MatxsMaterialData ***** //

//constructor
MatxsMaterialData::MatxsMaterialData(void)
{
  clear();
}

//destructor
MatxsMaterialData::~MatxsMaterialData(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMaterialData::clear()
{
  ngrp.clear();
  joutp.clear();
  nsubm = 0;

  //5d
  hmat.clear();

  amass = 0.0;

  temp.clear();
  sigz.clear();
  itype.clear();
  n1d.clear();
  n2d.clear();
  locs.clear();

  noutg.clear();

  //6-10d
  vector<MatxsSubmaterialData>().swap(submat_obj);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMaterialData::read_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsMaterialData";
  string func_name  = "read_data";
  vector<string> str_vec;

  if( nsubm == 0 )
  {
    vector<string> err_com;
    err_com.push_back("Please set nsubm value to read 5d data.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  if( static_cast<int>(text_vec[line_no].length()) <= 12 )
  {
    string str_data = "line data : " + text_vec[line_no];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("The number of characters in 5d data must be larger than 12.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  //Read Line No. 1
  str_vec.resize(3);
  str_vec[0] = text_vec[line_no].substr( 0, 4); // 5d
  str_vec[1] = text_vec[line_no].substr( 4, 8); //hmat
  str_vec[2] = text_vec[line_no].substr(12);    //amass
  str_vec[2] = str_obj.get_formatted_string_data(str_vec[2],12);

  if( str_vec[0].find("5d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 5d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }
  line_no++;
  str_obj.skip_no_space_string_data(text_vec, line_no);

  hmat  = str_vec[1];
  amass = numeric_string_cast<Real>(str_vec[2]);
  str_vec.clear();

  //Read Line No. 2
  temp.resize(nsubm);
  sigz.resize(nsubm);
  itype.resize(nsubm);
  n1d.resize(nsubm);
  n2d.resize(nsubm);
  locs.resize(nsubm);

  str_vec.resize(6);
  for(int i=0; i<nsubm; i++)
  {
    if( static_cast<int>(text_vec[line_no].length()) <= 42 )
    {
      ostringstream oss01, oss02;
      oss01 << i+1;
      oss01 << nsubm;

      string str_data01 = "line data : " + text_vec[line_no];
      string str_data02 = "nsubm     : " + oss01.str() + " / " + oss02.str();

      vector<string> err_com;
      err_com.push_back(str_data01);
      err_com.push_back(str_data02);
      err_com.push_back("The number of characters in 5d data must be larger than 42.");
      err_com.push_back("Please check the MATXS file.");
  
      err_obj.output_runtime_error(class_name, func_name, err_com);
    }

    temp[i]  = numeric_string_cast<Real>(text_vec[line_no].substr( 0,12));
    sigz[i]  = numeric_string_cast<Real>(text_vec[line_no].substr(12,12));
    itype[i] = numeric_string_cast<Integer>(text_vec[line_no].substr(24, 6));
    n1d[i]   = numeric_string_cast<Integer>(text_vec[line_no].substr(30, 6));
    n2d[i]   = numeric_string_cast<Integer>(text_vec[line_no].substr(36, 6));
    locs[i]  = numeric_string_cast<Integer>(text_vec[line_no].substr(42, 6));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  calc_noutg();

  //Read 6d - 10d data
  submat_obj.resize(nsubm);
  for(int i=0; i<nsubm; i++)
  {
    submat_obj[i].set_noutg(noutg[i]);
    submat_obj[i].set_n1d(n1d[i]);
    submat_obj[i].set_n2d(n2d[i]);
    submat_obj[i].read_data(text_vec, line_no);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMaterialData::calc_noutg()
{
  int i_max   = static_cast<int>(itype.size());
  int ngr_no  = static_cast<int>(ngrp.size());
  int jout_no = static_cast<int>(joutp.size());

  if( ngr_no == 0 || jout_no == 0 )
  {
    string class_name = "MatxsMaterialData";
    string func_name  = "calc_ng()";

    vector<string> err_com;
    err_com.push_back("Please set ngrp and joutp value to identify the number of energy grid.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  noutg.resize(i_max);
  for(int i=0; i<i_max; i++)
  {
    noutg[i] = 0;

    int i_pos = static_cast<int>(itype[i]);

    if( i_pos > jout_no )
    {
      string class_name = "MatxsMaterialData";
      string func_name  = "calc_noutg()";

      ostringstream oss01, oss02, oss03, oss04;
      oss01 << i+1;
      oss02 << i_max;
      oss03 << i_pos;
      oss04 << jout_no;
      string str_data01 = "Position             : " + oss01.str() + " / " + oss02.str();
      string str_data02 = "itype                : " + oss03.str();
      string str_data03 = "Data number of joutp : " + oss04.str();

      vector<string> err_com;
      err_com.push_back(str_data01);
      err_com.push_back(str_data02);
      err_com.push_back(str_data03);
      err_com.push_back("The itype value is larger than the data number of joutp.");
      err_com.push_back("This MATXS file may not be incorrect.");

      err_obj.output_runtime_error(class_name, func_name, err_com);
    }

    int e_pos = static_cast<int>(joutp[i_pos-1]);
    if( e_pos > ngr_no )
    {
      string class_name = "MatxsMaterialData";
      string func_name  = "calc_noutg()";

      ostringstream oss01, oss02, oss03, oss04, oss05;
      oss01 << i+1;
      oss02 << i_max;
      oss03 << i_pos;
      oss04 << e_pos;
      oss05 << ngr_no;
      string str_data01 = "Position            : " + oss01.str() + " / " + oss02.str();
      string str_data02 = "itype               : " + oss03.str();
      string str_data03 = "joutp               : " + oss04.str();
      string str_data04 = "Data number of ngrp : " + oss05.str();

      vector<string> err_com;
      err_com.push_back(str_data01);
      err_com.push_back(str_data02);
      err_com.push_back(str_data03);
      err_com.push_back("The joutp value is larger than the data number of ngrp.");
      err_com.push_back("This MATXS file may not be incorrect.");

      err_obj.output_runtime_error(class_name, func_name, err_com);
    }

    noutg[i] = ngrp[e_pos-1];
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<string> MatxsMaterialData::write_data()
{
  StringUtils str_obj;

  vector<string> text, str_vec;
  text.clear();
  str_vec.clear();

  calc_locs();

  string header = " 5d";
  str_vec.push_back(str_obj.get_formatted_string_data(header, 4));
  str_vec.push_back(str_obj.get_formatted_string_data(hmat, 8));
  str_vec.push_back(str_obj.get_formatted_real_data(amass,12));

  text.push_back(str_obj.get_merged_string_vector_data(str_vec));
  str_vec.clear();

  int i_max = static_cast<int>(temp.size());
  nsubm = i_max;
  for(int i=0; i<i_max; i++)
  {
    str_vec.push_back(str_obj.get_formatted_real_data(temp[i],12));
    str_vec.push_back(str_obj.get_formatted_real_data(sigz[i],12));
    str_vec.push_back(str_obj.get_formatted_int_data(itype[i],-6));
    str_vec.push_back(str_obj.get_formatted_int_data(n1d[i],  -6));
    str_vec.push_back(str_obj.get_formatted_int_data(n2d[i],  -6));
    str_vec.push_back(str_obj.get_formatted_int_data(locs[i], -6));

    text.push_back(str_obj.get_merged_string_vector_data(str_vec));
    str_vec.clear();
  }

  //Write 6d - 10d data
  for(int i=0; i<static_cast<int>(submat_obj.size()); i++)
  {
    if( n1d[i] > 0 || n2d[i] > 0 )
    {
      str_obj.add_string_vector_data(text, submat_obj[i].write_data());
    }
  }

  return text;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMaterialData::calc_locs()
{
  if( static_cast<int>(locs.size()) == static_cast<int>(submat_obj.size()) )
  {
    for(int i=0; i<static_cast<int>(submat_obj.size()); i++)
    {
      if( i==0 )
      {
        locs[0] = 0;
      }
      else
      {
        locs[i] = locs[i-1] + submat_obj[i-1].get_loc_no();
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Integer MatxsMaterialData::get_loc_no()
{
  Integer loc_no = 0;

  //Count 5d
  if( static_cast<int>(temp.size()) > 0 )
  {
    //loc_no++;
  }

  //Cout 6d - 10d
  for(int i=0; i<static_cast<int>(submat_obj.size()); i++)
  {
    loc_no += submat_obj[i].get_loc_no();
  }

  return loc_no;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Setter
void MatxsMaterialData::set_ngrp(vector<Integer> int_vec)
{
  ngrp = int_vec;
}
void MatxsMaterialData::set_joutp(vector<Integer> int_vec)
{
  joutp = int_vec;
}
void MatxsMaterialData::set_nsubm(Integer int_data)
{
  nsubm = int_data;
}

void MatxsMaterialData::set_hmat(string str_data)
{
  hmat = str_data;
}
void MatxsMaterialData::set_amass(Real real_data)
{
  amass = real_data;
}

void MatxsMaterialData::set_temp(vector<Real> real_vec)
{
  temp = real_vec;
}
void MatxsMaterialData::set_sigz(vector<Real> real_vec)
{
  sigz = real_vec;
}
void MatxsMaterialData::set_itype(vector<Integer> int_vec)
{
  itype = int_vec;
}
void MatxsMaterialData::set_n1d(vector<Integer> int_vec)
{
  n1d = int_vec;
}
void MatxsMaterialData::set_n2d(vector<Integer> int_vec)
{
  n2d = int_vec;
}
void MatxsMaterialData::set_locs(vector<Integer> int_vec)
{
  locs = int_vec;
}

//6-10d
void MatxsMaterialData::set_submat_obj(vector<MatxsSubmaterialData> obj_vec)
{
  submat_obj = obj_vec;
  vector<MatxsSubmaterialData>().swap(obj_vec);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Getter
vector<Integer> MatxsMaterialData::get_ngrp()
{
  return ngrp;
}
vector<Integer> MatxsMaterialData::get_joutp()
{
  return joutp;
}
Integer MatxsMaterialData::get_nsubm()
{
  return nsubm;
}

string MatxsMaterialData::get_hmat()
{
  return hmat;
}
Real   MatxsMaterialData::get_amass()
{
  return amass;
}

vector<Real>    MatxsMaterialData::get_temp()
{
  return temp;
}
vector<Real>    MatxsMaterialData::get_sigz()
{
  return sigz;
}
vector<Integer> MatxsMaterialData::get_itype()
{
  return itype;
}
vector<Integer> MatxsMaterialData::get_n1d()
{
  return n1d;
}
vector<Integer> MatxsMaterialData::get_n2d()
{
  return n2d;
}
vector<Integer> MatxsMaterialData::get_locs()
{
  calc_locs();
  return locs;
}

//6-10d
vector<MatxsSubmaterialData> MatxsMaterialData::get_submat_obj()
{
  return submat_obj;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

// ***** MatxsSubmaterialData ***** //

//constructor
MatxsSubmaterialData::MatxsSubmaterialData(void)
{
  clear();
}

//destructor
MatxsSubmaterialData::~MatxsSubmaterialData(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsSubmaterialData::clear()
{
  noutg = 0;
  n1d   = 0;
  n2d   = 0;

  //6d
  hvps.clear();
  nfg.clear();
  nlg.clear();

  //7d
  VectorClearer clr_obj;
  clr_obj.clear_vec_array2_real(vps);

  //8-10d
  vector<MatxsMatrixData>().swap(matrix_obj);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsSubmaterialData::read_data(vector<string>& text_vec, int& line_no)
{
  if( noutg == 0 && n1d == 0 && n2d == 0 )
  {
    string class_name = "MatxsSubmaterialData";
    string func_name  = "read_data";

    ostringstream oss01, oss02, oss03;
    oss01 << noutg;
    oss02 << n1d;
    oss03 << n2d;

    string str_data01 = "noutg : " + oss01.str();
    string str_data02 = "n1d   : " + oss02.str();
    string str_data03 = "n2d   : " + oss03.str();

    vector<string> err_com;
    err_com.push_back(str_data01);
    err_com.push_back(str_data02);
    err_com.push_back(str_data03);
    err_com.push_back("Please set noutg, n1d and n2d to read 6d and 7d data.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  if( n1d > 0 )
  {
    if( text_vec[line_no].find("6d") != string::npos )
    {
      read_6d_data(text_vec, line_no);
    }
    if( text_vec[line_no].find("7d") != string::npos )
    {
      read_7d_data(text_vec, line_no);
    }
  }

  //Read 8d - 10d data
  matrix_obj.resize(n2d);
  for(int i=0; i<n2d; i++)
  {
    matrix_obj[i].set_noutg(noutg);
    matrix_obj[i].read_data(text_vec, line_no);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsSubmaterialData::read_6d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsSubmaterialData";
  string func_name  = "read_6d_data";
  vector<string> str_vec;

  //Line No. 1
  int line_no_ori = line_no;
  int data_no     = n1d + 1;

  str_vec.clear();
  while( static_cast<int>(str_vec.size()) < data_no )
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    str_obj.add_string_vector_data(str_vec, str_obj.divide_string_data(text_vec[line_no], 8, 72));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  if( str_vec[0].find("6d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no_ori];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 6d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  hvps.resize(n1d);
  for(int i=1; i<data_no; i++)
  {
    hvps[i-1] = str_vec[i];
  }
  str_vec.clear();

  //Line No. 2
  data_no = 2*n1d;

  str_vec.clear();
  while( static_cast<int>(str_vec.size()) < data_no )
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    str_obj.add_string_vector_data(str_vec, str_obj.divide_string_data(text_vec[line_no], 6, 72));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  nfg.resize(n1d);
  nlg.resize(n1d);

  int ele_no = 0;
  for(int i=0; i<n1d; i++)
  {
    nfg[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  for(int i=0; i<n1d; i++)
  {
    nlg[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  str_vec.clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsSubmaterialData::read_7d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsSubmaterialData";
  string func_name  = "read_7d_data";
  vector<string> str_vec;

  //Line No. 1
  int line_no_ori = line_no;
  int data_no     = 1;
  vector<int> data_no_vec;

  vps.resize(n1d);
  for(int i=0; i<n1d; i++)
  {
    int ng2_no = nlg[i] - nfg[i] + 1;

    if( data_no + ng2_no > matxw_def )
    {
      data_no_vec.push_back(data_no);
      data_no = 1;
    }

    data_no += ng2_no;
    vps[i].resize(ng2_no);
  }

  if( data_no > 1 )
  {
    data_no_vec.push_back(data_no);
  }

  str_vec.clear();
  for(int i=0; i<static_cast<int>(data_no_vec.size()); i++)
  {
    vector<string> str_vec_each;
    while( static_cast<int>(str_vec_each.size()) < data_no_vec[i] )
    {
      str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

      str_obj.add_string_vector_data(str_vec_each, str_obj.divide_string_data(text_vec[line_no], 12, 72));
      line_no++;
    }

    if( i == 0 )
    {
      str_vec.push_back(str_vec_each[0]);
    }

    for(int j=1; j<data_no_vec[i]; j++)
    {
      str_vec.push_back(str_vec_each[j]);
    }
    str_vec_each.clear();
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  if( str_vec[0].find("7d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no_ori];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 7d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  int ele_no = 1;
  for(int i=0; i<static_cast<int>(vps.size()); i++)
  {
    for(int j=0; j<static_cast<int>(vps[i].size()); j++)
    {
      vps[i][j] = numeric_string_cast<Real>(str_vec[ele_no]);
      ele_no++;
    }
  }
  str_vec.clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<string> MatxsSubmaterialData::write_data()
{
  StringUtils str_obj;

  vector<string> text, str_vec;
  text.clear();
  str_vec.clear();

  //Write 6d data
  if( static_cast<int>(nfg.size()) > 0 )
  {
    //Line No. 1
    string header = " 6d";
    str_vec.push_back(str_obj.get_formatted_string_data(header, 8));
    for(int i=0; i<static_cast<int>(hvps.size()); i++)
    {
      str_vec.push_back(str_obj.get_formatted_string_data(hvps[i], 8));
    }
    str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 9));
    str_vec.clear();

    //Line No. 2
    for(int i=0; i<static_cast<int>(nfg.size()); i++)
    {
      str_vec.push_back(str_obj.get_formatted_int_data(nfg[i],-6));
    }

    for(int i=0; i<static_cast<int>(nlg.size()); i++)
    {
      str_vec.push_back(str_obj.get_formatted_int_data(nlg[i],-6));
    }
    str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 12));
    str_vec.clear();
  }

  //Write 7d data
  if( static_cast<int>(vps.size()) > 0 )
  {
    string header = " 7d";
    str_vec.push_back(str_obj.get_formatted_string_data(header, 12));
    for(int i=0; i<static_cast<int>(vps.size()); i++)
    {
      int j_max = static_cast<int>(vps[i].size());
      if( static_cast<int>(str_vec.size()) + j_max > matxw_def )
      {
        str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 6));
        str_vec.clear();

        str_vec.push_back(str_obj.get_formatted_string_data(header, 12));
      }

      for(int j=0; j<j_max; j++)
      {
        str_vec.push_back(str_obj.get_formatted_real_data(vps[i][j], 12));
      }
    }

    if( static_cast<int>(str_vec.size()) > 1 )
    {
      str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 6));
    }
    str_vec.clear();
  }

  //Write 8d - 10d data
  for(int i=0; i<static_cast<int>(matrix_obj.size()); i++)
  {
    str_obj.add_string_vector_data(text, matrix_obj[i].write_data());
  }

  return text;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Integer MatxsSubmaterialData::get_loc_no()
{
  Integer loc_no = 0;

  //Count 6d
  if( static_cast<int>(nfg.size()) > 0 )
  {
    loc_no++;
  }

  //Cout 7d
  if( static_cast<int>(vps.size()) > 0 )
  {
    int no_7d   = 0;
    int data_no = 1;
    for(int i=0; i<static_cast<int>(vps.size()); i++)
    {
      int j_max = static_cast<int>(vps[i].size());
      if( data_no + j_max > matxw_def )
      {
        no_7d++;
        data_no = 1 + j_max;
      }
      else
      {
        data_no += j_max;
      }
    }

    if( data_no > 1 )
    {
      no_7d++;
    }

    loc_no += no_7d;
  }
 
  //Cout 8d - 10d
  for(int i=0; i<static_cast<int>(matrix_obj.size()); i++)
  {
    loc_no += matrix_obj[i].get_loc_no();
  }

  return loc_no;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Setter
void MatxsSubmaterialData::set_noutg(Integer int_data)
{
  noutg = int_data;
}
void MatxsSubmaterialData::set_n1d(Integer int_data)
{
  n1d = int_data;
}
void MatxsSubmaterialData::set_n2d(Integer int_data)
{
  n2d = int_data;
}

//6d
void MatxsSubmaterialData::set_hvps(vector<string> str_vec)
{
  hvps = str_vec;
}
void MatxsSubmaterialData::set_nfg(vector<Integer> int_vec)
{
  nfg = int_vec;
}
void MatxsSubmaterialData::set_nlg(vector<Integer> int_vec)
{
  nlg = int_vec;
}

//7d
void MatxsSubmaterialData::set_vps(vector<vector<Real> > real_vec)
{
  vps = real_vec;
}

//8-10d
void MatxsSubmaterialData::set_matrix_obj(vector<MatxsMatrixData> obj_vec)
{
  matrix_obj = obj_vec;
  vector<MatxsMatrixData>().swap(obj_vec);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Getter
Integer MatxsSubmaterialData::get_noutg()
{
  return noutg;
}
Integer MatxsSubmaterialData::get_n1d()
{
  return n1d;
}
Integer MatxsSubmaterialData::get_n2d()
{
  return n2d;
}

//6d
vector<string>  MatxsSubmaterialData::get_hvps()
{
  return hvps;
}
vector<Integer> MatxsSubmaterialData::get_nfg()
{
  return nfg;
}
vector<Integer> MatxsSubmaterialData::get_nlg()
{
  return nlg;
}

//7d
vector<vector<Real> > MatxsSubmaterialData::get_vps()
{
  return vps;
}

//8-10d
vector<MatxsMatrixData> MatxsSubmaterialData::get_matrix_obj()
{
  return matrix_obj;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

// ***** MatxsMatrixData ***** //

//constructor
MatxsMatrixData::MatxsMatrixData(void)
{
  clear();
}

//destructor
MatxsMatrixData::~MatxsMatrixData(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMatrixData::clear()
{
  noutg = 0;

  //8d
  hmtx.clear();
  lord   = 0;
  jconst = 0;
  jband.clear();
  ijj.clear();

  //9d
  VectorClearer clr_obj;
  clr_obj.clear_vec_array3_real(scat);

  //10d
  spec.clear();
  prod.clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMatrixData::read_data(vector<string>& text_vec, int& line_no)
{
  if( noutg == 0 )
  {
    string class_name = "MatxsMatrixData";
    string func_name  = "read_data";

    vector<string> err_com;
    err_com.push_back("Please set noutg to read 8d - 10d data.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  if( text_vec[line_no].find("8d") != string::npos )
  {
    read_8d_data(text_vec, line_no);
  }
  if( text_vec[line_no].find("9d") != string::npos )
  {
    read_9d_data(text_vec, line_no);
  }
  if( jconst > 0 )
  {
    read_10d_data(text_vec, line_no);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMatrixData::read_8d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsMatrixData";
  string func_name  = "read_8d_data";
  vector<string> str_vec;

  //Line No. 1
  str_vec = str_obj.divide_string_data(text_vec[line_no], 8, 16);

  if( str_vec[0].find("8d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 8d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }
  line_no++;
  str_obj.skip_no_space_string_data(text_vec, line_no);

  hmtx = str_vec[1];
  str_vec.clear();

  //Line No. 2
  int data_no = 2 + 2*noutg;
  while( static_cast<int>(str_vec.size()) < data_no )
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    str_obj.add_string_vector_data(str_vec, str_obj.divide_string_data(text_vec[line_no], 6, 72));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  lord   = numeric_string_cast<Integer>(str_vec[0]);
  jconst = numeric_string_cast<Integer>(str_vec[1]);

  jband.resize(noutg);
  ijj.resize(noutg);

  int ele_no = 2;
  for(int i=0; i<noutg; i++)
  {
    jband[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
  for(int i=0; i<noutg; i++)
  {
    ijj[i] = numeric_string_cast<Integer>(str_vec[ele_no]);
    ele_no++;
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMatrixData::read_9d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsMatrixData";
  string func_name  = "read_9d_data";
  vector<string> str_vec;

  int line_no_ori = line_no;
  int data_no     = 1;
  vector<int> data_no_vec;

  for(int i=0; i<static_cast<int>(jband.size()); i++)
  {

    if( data_no + jband[i] * lord > matxw_def )
    {
      data_no_vec.push_back(data_no);
      data_no = 1;
    }

    data_no += jband[i] * lord;
  }

  if( data_no > 1 )
  {
    data_no_vec.push_back(data_no);
  }
  else
  {
    VectorClearer clr_obj;
    clr_obj.clear_vec_array3_real(scat);

    line_no++;
    return;
  }

  str_vec.clear();
  for(int i=0; i<static_cast<int>(data_no_vec.size()); i++)
  {
    vector<string> str_vec_each;
    while( static_cast<int>(str_vec_each.size()) < data_no_vec[i] )
    {
      str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

      str_obj.add_string_vector_data(str_vec_each, str_obj.divide_string_data(text_vec[line_no], 12, 72));
      line_no++;
    }

    if( i == 0 )
    {
      str_vec.push_back(str_vec_each[0]);
    }

    for(int j=1; j<data_no_vec[i]; j++)
    {
      str_vec.push_back(str_vec_each[j]);
    }
    str_vec_each.clear();
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  if( str_vec[0].find("9d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no_ori];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 9d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  int ele_no = 1;
  int i_max  = static_cast<int>(jband.size());
  scat.resize(i_max);
  for(int i=0; i<i_max; i++)
  {
    int j_max = static_cast<int>(lord);
    scat[i].resize(j_max);
    for(int j=0; j<j_max; j++)
    {
      int k_max = static_cast<int>(jband[i]);
      scat[i][j].resize(k_max);
      for(int k=0; k<k_max; k++)
      {
        scat[i][j][k] = numeric_string_cast<Real>(str_vec[ele_no]);
        ele_no++;
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void MatxsMatrixData::read_10d_data(vector<string>& text_vec, int& line_no)
{
  StringUtils str_obj;
  string class_name = "MatxsMatrixData";
  string func_name  = "read_10d_data";
  vector<string> str_vec;

  int line_no_ori = line_no;
  int data_no     = 1 + noutg + jconst;

  str_vec.clear();
  while( static_cast<int>(str_vec.size()) < data_no )
  {
    str_obj.check_string_vector_data_end(text_vec, line_no, class_name, func_name);

    str_obj.add_string_vector_data(str_vec, str_obj.divide_string_data(text_vec[line_no], 12, 72));
    line_no++;
  }
  str_obj.skip_no_space_string_data(text_vec, line_no);

  if( str_vec[0].find("10d") == string::npos )
  {
    string str_data = "line data : " + text_vec[line_no_ori];

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("This data is not 10d data in MATXS format.");
    err_com.push_back("Please check the MATXS file.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  int ele_no = 1;
  spec.resize(noutg);
  prod.resize(jconst);
  for(int i=0; i<noutg; i++)
  {
    spec[i] = numeric_string_cast<Real>(str_vec[ele_no]);
    ele_no++;
  } 
  for(int i=0; i<jconst; i++)
  {
    prod[i] = numeric_string_cast<Real>(str_vec[ele_no]);
    ele_no++;
  } 
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<string> MatxsMatrixData::write_data()
{
  StringUtils str_obj;

  vector<string> text, str_vec;
  text.clear();
  str_vec.clear();

  //Write 8d data
  if( static_cast<int>(jband.size()) > 0 )
  {
    //Line No. 1
    string header = " 8d";
    str_vec.push_back(str_obj.get_formatted_string_data(header, 8));
    str_vec.push_back(str_obj.get_formatted_string_data(hmtx,   8));
    text.push_back(str_obj.get_merged_string_vector_data(str_vec));
    str_vec.clear();

    //Line No. 2
    str_vec.push_back(str_obj.get_formatted_int_data(lord,  -6));
    str_vec.push_back(str_obj.get_formatted_int_data(jconst,-6));
    for(int i=0; i<static_cast<int>(jband.size()); i++)
    {
      str_vec.push_back(str_obj.get_formatted_int_data(jband[i],-6));
    }
    for(int i=0; i<static_cast<int>(ijj.size()); i++)
    {
      str_vec.push_back(str_obj.get_formatted_int_data(ijj[i],-6));
    }
    str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 12));
    str_vec.clear();
  }

  //Write 9d data
  if( static_cast<int>(scat.size()) > 0 )
  {
    string header = " 9d";
    str_vec.push_back(str_obj.get_formatted_string_data(header, 12));
    for(int i=0; i<static_cast<int>(scat.size()); i++)
    {
      int j_max = static_cast<int>(scat[i].size());
      if( j_max == 0 )
      {
        continue;
      }

      int k_max = static_cast<int>(scat[i][0].size());
      if( static_cast<int>(str_vec.size()) + j_max * k_max > matxw_def )
      {
        str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 6));
        str_vec.clear();

        str_vec.push_back(str_obj.get_formatted_string_data(header, 12));
      }
      
      for(int j=0; j<j_max; j++)
      {
        for(int k=0; k<k_max; k++)
        {
          str_vec.push_back(str_obj.get_formatted_real_data(scat[i][j][k], 12));
        }
      }
    }

    if( static_cast<int>(str_vec.size()) > 1 )
    {
      str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 6));
    }
    str_vec.clear();
  }
  else if( static_cast<int>(jband.size()) > 0 )
  {
    string header = " 9d";
    str_vec.push_back(str_obj.get_formatted_string_data(header, 12));
    str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 6));
  }

  //Write 10d data
  if( static_cast<int>(spec.size()) > 0 )
  {
    string header = "10d";
    str_vec.push_back(str_obj.get_formatted_string_data(header, 12));
    for(int i=0; i<static_cast<int>(spec.size()); i++)
    {
      str_vec.push_back(str_obj.get_formatted_real_data(spec[i], 12));
    }
    for(int i=0; i<static_cast<int>(prod.size()); i++)
    {
      str_vec.push_back(str_obj.get_formatted_real_data(prod[i], 12));
    }
    str_obj.add_string_vector_data(text, str_obj.get_merged_string_vector_data(str_vec, 6));
    str_vec.clear();  
  }

  return text; 
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Integer MatxsMatrixData::get_loc_no()
{
  Integer loc_no = 0;

  //Count 8d
  if( static_cast<int>(jband.size()) > 0 )
  {
    loc_no++;
  }

  //Cout 9d
  if( static_cast<int>(scat.size()) > 0 )
  {
    int no_9d   = 0;
    int data_no = 1;
    for(int i=0; i<static_cast<int>(scat.size()); i++)
    {
      int j_max = static_cast<int>(scat[i].size());
      if( j_max > 0 )
      {
        int k_max = static_cast<int>(scat[i][0].size());
 
        if( data_no + j_max*k_max > matxw_def )
        {
          no_9d++;
          data_no = 1 + j_max*k_max;
        }
        else
        {
          data_no += j_max*k_max;
        }
      }
    }

    if( data_no > 1 )
    {
      no_9d++;
    }

    loc_no += no_9d;
  }
  else if( static_cast<int>(jband.size()) > 0 )
  {
    loc_no++;
  }
 
  //Cout 10d
  if( static_cast<int>(spec.size()) > 0 )
  {
    loc_no++;
  }

  return loc_no;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Setter
void MatxsMatrixData::set_noutg(Integer int_data)
{
  noutg = int_data;
}

//8d
void MatxsMatrixData::set_hmtx(string str_data)
{
  hmtx = str_data;
}
void MatxsMatrixData::set_lord(Integer int_data)
{
  lord = int_data;
}
void MatxsMatrixData::set_jconst(Integer int_data)
{
  jconst = int_data;
}
void MatxsMatrixData::set_jband(vector<Integer> int_vec)
{
  jband = int_vec;
}
void MatxsMatrixData::set_ijj(vector<Integer> int_vec)
{
  ijj = int_vec;
}

//9d
void MatxsMatrixData::set_scat(vector<vector<vector<Real> > > real_vec)
{
  scat = real_vec;
}

//10d
void MatxsMatrixData::set_spec(vector<Real> real_vec)
{
  spec = real_vec;
}
void MatxsMatrixData::set_prod(vector<Real> real_vec)
{
  prod = real_vec;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Getter
Integer         MatxsMatrixData::get_noutg()
{
  return noutg;
}

//8d
string          MatxsMatrixData::get_hmtx()
{
  return hmtx;
}
Integer         MatxsMatrixData::get_lord()
{
  return lord;
}
Integer         MatxsMatrixData::get_jconst()
{
  return jconst;
}
vector<Integer> MatxsMatrixData::get_jband()
{
  return jband;
}
vector<Integer> MatxsMatrixData::get_ijj()
{
  return ijj;
}

//9d
vector<vector<vector<Real> > > MatxsMatrixData::get_scat()
{
  return scat;
}

//10d
vector<Real> MatxsMatrixData::get_spec()
{
  return spec;
}
vector<Real> MatxsMatrixData::get_prod()
{
  return prod;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

