#include "NuclearDataModifier/Endf6ModifierByFrendyInput.hpp"

using namespace frendy;

//constructor
Endf6ModifierByFrendyInput::Endf6ModifierByFrendyInput(void)
{
  clear();
}

//destructor
Endf6ModifierByFrendyInput::~Endf6ModifierByFrendyInput(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::clear()
{
  calc_flg = 0;

  frendy_inp_obj.clear();

  nucl_data_obj_ori.clear();
  nucl_data_obj_add.clear();
  nucl_data_obj_mod.clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::calc_endf6_modifier()
{
  if( calc_flg != 0 )
  {
    return;
  }

  calc_flg = 1;

  cout << endl;
  cout << "======================================================================" << endl;
  cout << endl;
  cout << "+++ Start ENDF file modification +++" << endl;
  cout << endl;

  int i_max = static_cast<int>(frendy_inp_obj.get_run_flg().size());
  for(int i=0; i<i_max; i++)
  {
    int     run_flg = frendy_inp_obj.get_run_flg()[i];
    Integer mf_no   = frendy_inp_obj.get_mf_no()[i];
    Integer mt_no   = frendy_inp_obj.get_mt_no()[i];

    string endf_name_ori = frendy_inp_obj.get_endf_name_ori()[i];
    string endf_name_add = frendy_inp_obj.get_endf_name_add()[i];
    string endf_name_mod = frendy_inp_obj.get_endf_name_mod()[i];

    int run_mode_flg    = run_flg / 100;
    int target_mode_flg = run_flg % 100;
    frendy_inp_obj.output_input_information(run_mode_flg, target_mode_flg, mf_no, mt_no,
                                            endf_name_ori, endf_name_add, endf_name_mod, i);

    modify_endf6_format(run_flg, mf_no, mt_no, endf_name_ori, endf_name_add, endf_name_mod);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::modify_endf6_format
                                   (int run_flg, Integer mf_no, Integer mt_no,
                                    string endf_name_ori_inp, string endf_name_add_inp, string endf_name_mod_inp)
{
  int run_mode_flg    = run_flg / 100;
  int target_mode_flg = run_flg % 100;

  nucl_data_obj_ori = read_obj.get_nucl_data_obj(endf_name_ori_inp);
  nucl_data_obj_mod = nucl_data_obj_ori;

  if( run_mode_flg == frendy_inp_obj.remove_data_mode )
  {
    remove_endf6_data(target_mode_flg, mf_no, mt_no, nucl_data_obj_mod);
  }
  else if( run_mode_flg == frendy_inp_obj.change_data_mode )
  {
    nucl_data_obj_add = read_obj.get_nucl_data_obj(endf_name_add_inp);
    change_endf6_data(target_mode_flg, mf_no, mt_no, nucl_data_obj_mod, nucl_data_obj_add);
  }
  else if( run_mode_flg == frendy_inp_obj.add_data_mode )
  {
    nucl_data_obj_add = read_obj.get_nucl_data_obj(endf_name_add_inp);
    add_endf6_data(target_mode_flg, mf_no, mt_no, nucl_data_obj_mod, nucl_data_obj_add);
  }

  endf_wri_obj.set_nucl_data_obj(nucl_data_obj_mod);
  endf_wri_obj.write_pendf_with_tape_end(endf_name_mod_inp);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::remove_endf6_data
                                   (int target_mode_flg, Integer mf_no, Integer mt_no,
                                    NuclearDataObject& nucl_data_obj)
{
  if( target_mode_flg == frendy_inp_obj.mt_mode )
  {
    nucl_mod_obj.remove_specified_reaction_type_data(nucl_data_obj, mt_no);
  }
  else if( target_mode_flg == frendy_inp_obj.mf_mode )
  {
    endf_mod_obj.remove_specified_MF_data(nucl_data_obj, mf_no);
  }
  else if( target_mode_flg == frendy_inp_obj.mf_mt_mode )
  {
    endf_mod_obj.remove_specified_MF_MT_data(nucl_data_obj, mf_no, mt_no);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::change_endf6_data
                                   (int target_mode_flg, Integer mf_no, Integer mt_no,
                                    NuclearDataObject& nucl_data_obj, NuclearDataObject& nucl_data_obj_add)
{
  if( target_mode_flg == frendy_inp_obj.mt_mode )
  {
    nucl_mod_obj.change_specified_reaction_type_data(nucl_data_obj, nucl_data_obj_add, mt_no);
  }
  else if( target_mode_flg == frendy_inp_obj.mf_mode )
  {
    endf_mod_obj.change_specified_MF_data(nucl_data_obj, nucl_data_obj_add, mf_no);
  }
  else if( target_mode_flg == frendy_inp_obj.mf_mt_mode )
  {
    endf_mod_obj.change_specified_MF_MT_data(nucl_data_obj, nucl_data_obj_add, mf_no, mt_no);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::add_endf6_data
                                   (int target_mode_flg, Integer mf_no, Integer mt_no,
                                    NuclearDataObject& nucl_obj, NuclearDataObject& nucl_obj_add)
{
  if( target_mode_flg == frendy_inp_obj.mt_mode )
  {
    nucl_mod_obj.add_specified_reaction_type_data(nucl_obj, nucl_obj_add, mt_no);
  }
  else if( target_mode_flg == frendy_inp_obj.mf_mode )
  {
    endf_mod_obj.add_specified_MF_data(nucl_obj, nucl_obj_add, mf_no);
  }
  else if( target_mode_flg == frendy_inp_obj.mf_mt_mode )
  {
    endf_mod_obj.add_specified_MF_MT_data(nucl_obj, nucl_obj_add, mf_no, mt_no);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::set_frendy_input(string input_file_name)
{
  calc_flg = 0;
  frendy_inp_obj.read_input_data_from_input_file(input_file_name);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void Endf6ModifierByFrendyInput::set_frendy_inp_obj(Endf6ModifierInputReader inp_obj)
{
  calc_flg = 0;
  frendy_inp_obj = inp_obj;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Endf6ModifierInputReader Endf6ModifierByFrendyInput::get_frendy_inp_obj()
{
  return frendy_inp_obj;
}

NuclearDataObject Endf6ModifierByFrendyInput::get_nucl_data_obj_ori()
{
  calc_endf6_modifier();
  return nucl_data_obj_ori;
}

NuclearDataObject Endf6ModifierByFrendyInput::get_nucl_data_obj_add()
{
  calc_endf6_modifier();
  return nucl_data_obj_add;
}

NuclearDataObject Endf6ModifierByFrendyInput::get_nucl_data_obj_mod()
{
  calc_endf6_modifier();
  return nucl_data_obj_mod;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Real Endf6ModifierByFrendyInput::get_time_total()
{
  return time_total;
}

Real Endf6ModifierByFrendyInput::get_calc_time()
{
  return time_total;
}

