#include "ReconResonance/RmatrixLimited/endf/LTT1.h"
#include "ReconResonance/RmatrixLimited/endf/Construct.h"

////ClassImp( LTT1::Record );
////ClassImp( LTT1 );


LTT1::Record:: Record() {}
LTT1::Record:: Record( const LIST& o ) { this->Set( o ); }
LTT1::Record::~Record() {}


/*! @brief Legendre coefficient
@param[in] l : angular momentum
*/
double LTT1::Record::a( const int l ) const {

  if( l > this->NL() ) return 0.0;

  return ( l == 0 ) ? 1.0 : this->B(l-1);

}

/*! @brief Normalized distribution */
/*
double LTT1::Record::f( const double cosangle ) const {

  double f = 0.0;

    for( int l=0; l<=this->NL(); l++ )

      f += 0.5*(2.0*l+1.0) * this->a(l) * P_Legendre( l, cosangle );

  return f;

}
*/


/*! @brief Reconstruct normalized distribution from the Legendre polynomial expression
@param[out] f         : normalized distribution
@param[in] interpcode : interpolation code for the distribution
@param[in] eps        : accuracy of the interpolation
*/


LTT1:: LTT1(){}
LTT1:: LTT1( const int n, const int interpcode ) { this->Set( n, interpcode ); }
LTT1::~LTT1(){}

LTT1::Record  LTT1::at( const int i ) const { return array.at(i); }
LTT1::Record& LTT1::at( const int i )       { return array.at(i); }

LTT1::Record  LTT1::operator[]( const int i ) const { return array.operator[](i); }
LTT1::Record& LTT1::operator[]( const int i )       { return array.operator[](i); }

int  LTT1::size () const { return array.size (); }
bool LTT1::empty() const { return array.empty(); }

LTT1::Record  LTT1::Front() const { return array.front(); }
LTT1::Record& LTT1::Front()       { return array.front(); }
LTT1::Record  LTT1::Back () const { return array.back (); }
LTT1::Record& LTT1::Back ()       { return array.back (); }



/*! @brief Read data from a stream */
void LTT1::Store( istream& is ) {

  return store( this, &array, is );

}

/*! @brief Read data from a file */
void LTT1::Read( const char* file_name ) {

  return read( this, &array, file_name );

}

/*! @brief Write data to a stream */
void LTT1::Write( ostream& os, const int MAT, const int MF, const int MT, int* NS ) const {

  write( this, &array, os, MAT,MF,MT,NS );

}

/*! @brief Write data to a file */
void LTT1::Write( const char* file_name ) const {

  return write( this, &array, file_name );

}

/*! @brief Count number of lines in the file */
unsigned long LTT1::NumberOfLines() const {

  return number_of_lines( this, &array );

}

/*! @brief Set data */
LTT1* LTT1::Set( const LTT1& o ) {

  set_array( this, &array, o, o.array );  return this;

}

/*! @brief Set data from a file ( same as Write ) */
LTT1* LTT1::Set( const char* file_name ) {

  this->Read( file_name );  return this;

}

/*! @brief Resize array
@param[in] n           : number of the record
@param[in] interpcode  : interpolation code between the records
*/
LTT1* LTT1::Set( const int n, const int interpcode ) {

  set_array( this, &array, n, interpcode );  return this;

}

/*! @brief Clear array */
LTT1* LTT1::Clear() {

  clear( this, &array );  return this;

}

/*! @brief Push a record from the front */
LTT1* LTT1::PushFront( const Record& o, const int interpcode ) {

  push_front( this, &array, o, interpcode );  return this;

}

/*! @brief Push a record from the back */
LTT1* LTT1::PushBack ( const Record& o, const int interpcode ) {

  push_back( this, &array, o, interpcode );  return this;

}

/*! @brief Push a record from the front */
LTT1* LTT1::PushFront( const LTT1& o, const int interpcode ) {

  push_front( this, &array, o, o.array, interpcode );  return this;

}

/*! @brief Push a record from the back */
LTT1* LTT1::PushBack ( const LTT1& o, const int interpcode ) {

  push_back( this, &array, o, o.array, interpcode );  return this;

}

/*! @brief Put a record outside the covering energy range,
where the new record will be taken from the first/last node
*/
LTT1* LTT1::PutClone( const double elab ) {

  put_clone( this, &array, elab );  return this;

}

/*! @brief Erase records from the front
@param[in] n : number of the records to be erased
*/
LTT1* LTT1::PopFront( const int num ) {

  pop_front( this, &array, num );  return this;

}

/*! @brief Erase records from the back
@param[in] n : number of the records to be erased
*/
LTT1* LTT1::PopBack ( const int num ) {

  pop_back( this, &array, num );  return this;

}

/*! @brief Erase a record */
LTT1* LTT1::Erase( const double E ) {

  erase( this, &array, E );  return this;

}

/*! @brief Cut all the records below E=elab
@param[in] elab : incident energy
@param[in] eps  : accuracy of the interpolation
@note a record will be generated by the interpolation if it is missing at elab
*/
LTT1* LTT1::CutBelow( const double elab, const double eps ) {

  cut_below( this, &array, elab );  if( this->empty() ) return this;

  if( this->Front().E() < elab ) this->Front() = this->Interp( elab );

  return this;

}

/*! @brief Cut all the records above E=elab
@param[in] elab : incident energy
@param[in] eps  : accuracy of the interpolation
@note a record will be generated by the interpolation if it is missing at elab
*/
LTT1* LTT1::CutAbove( const double elab, const double eps ) {

  cut_above( this, &array, elab );  if( this->empty() ) return this;

  if( this->Back().E() > elab ) this->Back() = this->Interp( elab );

  return this;

}


/*! @brief Replace data with new records for the given energy range */
LTT1* LTT1::Replace( const LTT1& o ) {

  if( o.Back ().E() < this->Front().E() ) return this;
  if( o.Front().E() > this->Back ().E() ) return this;

  LTT1 lower(*this),upper(*this);

    lower.CutAbove( o.Front().E() );
    upper.CutBelow( o.Back ().E() );

    this->Clear();

    this->PushBack( lower );
    this->PushBack(     o );
    this->PushBack( upper );

    if( eq( this->E(      0 ), this->E(      1 ) ) ) this->PopFront();
    if( eq( this->E( NE()-2 ), this->E( NE()-1 ) ) ) this->PopBack ();

/*
lower.Write(); cerr << endl;
    o.Write(); cerr << endl;
upper.Write(); cerr << endl;
*/
  return this;

}


/*! @brief Check if a record exist at E */
bool LTT1::Find( const double E ) const { return find( this, &array, E ); }

/*! @brief Check if given energy is inside the range */
bool LTT1::Seek( const double E ) const { return seek( this, &array, E ); }

/*! @brief Check if given energy is inside the range
@param[in]  E          : incident energy
@param[out] index      : index of the record where E should be in-between E(index) and E(index+1)
@param[out] interpcode : interpolation code between the two records
*/
bool LTT1::Seek( const double E, int* index, int* interpcode ) const {

  return seek( this, &array, E, index, interpcode );

}

//static double get_a( const LIST& o, const int l ) {
//
//  if( l > o.NPL() ) return 0.0;
//
//  return ( l == 0 ) ? 1.0 : o.B(l-1);
//
//}

LTT1* LTT1::ScaleEnergy( const double fac ) {

  for( int i=0; i<this->NE(); i++ ) this->E(i) *= fac;  return this;

}



/*! @brief Legendre polynomial coefficient
@param[in] elab : incident energy
@param[in] l    : order of the Legendre polynomial
*/
double LTT1::a( const double elab, const int l ) const {

  // exactly corresponding data

  for( int i=0; i<this->NE(); i++ ) if( eq( this->E(i), elab ) ) return array[i].a(l);

  // interp.

  int index, interpcode;

  if( ! this->Seek( elab, &index, &interpcode ) ) return 0.0;

  return Interp1( E(index), array[index].a(l), E(index+1), array[index+1].a(l), interpcode%20, elab );

}

double LTT1::a( const double elab_lower, const double elab_upper, const int l ) const {

  TAB1 b;

    for( int i=0; i<this->NE(); i++ ) b.PushBack( this->E(i), array[i].a(l), this->InterpCode(i-1) );

  return b.Average( elab_lower,elab_upper );

}


void LTT1::Interp( Record* result ) const {

  double elab = result->E();

  // exactly corresponding data

  for( int i=0; i<this->NE(); i++ ) if( eq( this->E(i), elab ) ) {

      *result = this->at(i);  return;

  }

  // interp.

  int index, interpcode;

  if( ! this->Seek( elab, &index, &interpcode ) ) return;

  int NL1 = this->NL(index  );
  int NL2 = this->NL(index+1);

  *result = NL1 > NL2 ? this->at(index) : this->at(index+1);

  for( int i=0; i<min(NL1,NL2); i++ )

    result->B(i) = Interp1( E(index), array[index].B(i), E(index+1), array[index+1].B(i), interpcode%20, elab );

  for( int i=min(NL1,NL2); i<max(NL1,NL2); i++ ) {

    if( NL1 < NL2 ) result->B(i) = Interp1( E(index),            0.0, E(index+1), array[index+1].B(i), interpcode%20, elab );
    if( NL1 > NL2 ) result->B(i) = Interp1( E(index), array[index].B(i), E(index+1),              0.0, interpcode%20, elab );

  }

  result->T() = Interp1( E(index), T(index), E(index+1), T(index+1), interpcode%20, elab );
  result->E() = elab;

  return;

}


LTT1::Record LTT1::Interp( const double elab ) const {

  Record a;  this->Interp( a.E(elab) );  return a;

}



/*! @brief Value of normalized distribution at given angle
@param[in] elab     : incident energy
@param[in] cosangle : cosine of the scattering angle
*/

