#include "ReconResonance/RmatrixLimited/endf/LTT2.h"


////ClassImp( LTT2::Record );
////ClassImp( LTT2 );


LTT2::Record:: Record() {}
LTT2::Record:: Record( const TAB1& o ) { this->Set( o ); }
LTT2::Record::~Record() {}

LTT2:: LTT2(){}
LTT2:: LTT2( const int n, const int interpcode ) { this->Set( n, interpcode ); }
LTT2::~LTT2(){}

LTT2::Record  LTT2::at( const int i ) const { return array.at(i); }
LTT2::Record& LTT2::at( const int i )       { return array.at(i); }

LTT2::Record  LTT2::operator[]( const int i ) const { return array.operator[](i); }
LTT2::Record& LTT2::operator[]( const int i )       { return array.operator[](i); }

int  LTT2::size () const { return array.size (); }
bool LTT2::empty() const { return array.empty(); }

LTT2::Record  LTT2::Front() const { return array.front(); }
LTT2::Record& LTT2::Front()       { return array.front(); }
LTT2::Record  LTT2::Back () const { return array.back (); }
LTT2::Record& LTT2::Back ()       { return array.back (); }



/*! @brief Read data from a stream */
void LTT2::Store( istream& is ) {

  return store( this, &array, is );

}

/*! @brief Read data from a file */
void LTT2::Read( const char* file_name ) {

  return read( this, &array, file_name );

}

/*! @brief Write data to a stream */
void LTT2::Write( ostream& os, const int MAT, const int MF, const int MT, int* NS ) const {

  write( this, &array, os, MAT,MF,MT,NS );

}

/*! @brief Write data to a file */
void LTT2::Write( const char* file_name ) const {

  return write( this, &array, file_name );

}

/*! @brief Count number of lines in the file */
unsigned long LTT2::NumberOfLines() const {

  return number_of_lines( this, &array );

}

/*! @brief Set data */
LTT2* LTT2::Set( const LTT2& o ) {

  set_array( this, &array, o, o.array );  return this;

}

/*! @brief Set data from a file ( same as Write ) */
LTT2* LTT2::Set( const char* file_name ) {

  this->Read( file_name );  return this;

}

/*! @brief Resize array
@param[in] n           : number of the record
@param[in] interpcode  : interpolation code between the records
*/
LTT2* LTT2::Set( const int n, const int interpcode ) {

  set_array( this, &array, n, interpcode );  return this;

}

/*! @brief Clear array */
LTT2* LTT2::Clear() {

  clear( this, &array );  return this;

}

/*! @brief Push a record from the front */
LTT2* LTT2::PushFront( const Record& o, const int interpcode ) {

  push_front( this, &array, o, interpcode );  return this;

}

/*! @brief Push a record from the back */
LTT2* LTT2::PushBack ( const Record& o, const int interpcode ) {

  push_back( this, &array, o, interpcode );  return this;

}

/*! @brief Push a record from the front */
LTT2* LTT2::PushFront( const LTT2& o, const int interpcode ) {

  push_front( this, &array, o, o.array, interpcode );  return this;

}

/*! @brief Push a record from the back */
LTT2* LTT2::PushBack ( const LTT2& o, const int interpcode ) {

  push_back( this, &array, o, o.array, interpcode );  return this;

}

/*! @brief Put a record outside the covering energy range,
where the new record will be taken from the first/last node
*/
LTT2* LTT2::PutClone( const double elab ) {

  put_clone( this, &array, elab );  return this;

}

/*! @brief Erase records from the front
@param[in] n : number of the records to be erased
*/
LTT2* LTT2::PopFront( const int num ) {

  pop_front( this, &array, num );  return this;

}

/*! @brief Erase records from the back
@param[in] n : number of the records to be erased
*/
LTT2* LTT2::PopBack ( const int num ) {

  pop_back( this, &array, num );  return this;

}

/*! @brief Erase a record */
LTT2* LTT2::Erase( const double E ) {

  erase( this, &array, E );  return this;

}

/*! @brief Cut all the records below E=elab
@param[in] elab : incident energy
@param[in] eps  : accuracy of the interpolation
@note a record will be generated by the interpolation if it is missing at elab
*/
LTT2* LTT2::CutBelow( const double elab, const double eps ) {

  cut_below( this, &array, elab );  if( this->empty() ) return this;

  if( this->Front().E() < elab ) this->Front() = this->Interp( elab, eps );

  return this;

}

/*! @brief Cut all the records above E=elab
@param[in] elab : incident energy
@param[in] eps  : accuracy of the interpolation
@note a record will be generated by the interpolation if it is missing at elab
*/
LTT2* LTT2::CutAbove( const double elab, const double eps ) {

  cut_above( this, &array, elab );  if( this->empty() ) return this;

  if( this->Back().E() > elab ) this->Back() = this->Interp( elab, eps );

  return this;

}


/*! @brief Check if a record exist at E */
bool LTT2::Find( const double E ) const { return find( this, &array, E ); }

/*! @brief Check if given energy is inside the range */
bool LTT2::Seek( const double E ) const { return seek( this, &array, E ); }

/*! @brief Check if given energy is inside the range
@param[in]  E          : incident energy
@param[out] index      : index of the record where E should be in-between E(index) and E(index+1)
@param[out] interpcode : interpolation code between the two records
*/
bool LTT2::Seek( const double E, int* index, int* interpcode ) const {

  return seek( this, &array, E, index, interpcode );

}



LTT2* LTT2::ScaleEnergy( const double fac ) {

  for( int i=0; i<this->NE(); i++ ) this->E(i) *= fac;  return this;

}




void LTT2::Interp( Record* result, const double eps ) const {

  // exactly corresponding data

  for( int i=0; i<this->NE(); i++ ) if( this->E(i) == result->E() ) {

    *result = this->at(i);  return;
    
  }

  // interp.

  int index, interpcode;

    if( ! this->Seek( result->E(), &index, &interpcode ) ) return;

    Interp2( this->at(index), this->at(index+1), interpcode, result, eps );

    result->T() = Interp1( E(index), T(index), E(index+1), T(index+1), interpcode%20, result->E() );

  return;

}


void LTT2::Interp( TAB1* result, const double eps ) const {

  Record a;

    this->Interp( a.E( result->C2() ) );  result->Set( a );

  return;

}

LTT2::Record LTT2::Interp( const double elab, const double eps ) const {

  Record a;  this->Interp( a.E(elab), eps );  return a;

}

/*! @brief Y is weighted by X^power ( default: Y -> XY ) */
LTT2* LTT2::Weight( const double power ) {

  for( int i=0; i<this->size(); i++ ) this->at(i).Weight( power );  return this;

}

/*! @brief Average distribution (tentative) */
LTT2* LTT2::Average( const int ndiv ) {

  for( int i=0; i<this->size(); i++ ) this->at(i).Average( ndiv );  return this;

}



/*! @brief Value of normalized distribution at given angle
@param[in] elab     : incident energy
@param[in] cosangle : cosine of the scattering angle
*/
double LTT2::f( const double elab, const double cosangle ) const {

  // exactly corresponding data

  for( int i=0; i<this->NE(); i++ ) if( eq( this->E(i), elab ) ) return this->at(i).Interp( cosangle );

  // interp.

  int index, interpcode;

  if( ! this->Seek( elab, &index, &interpcode ) ) return 0.0;

    double E1 = this->E (index  );
    double E2 = this->E (index+1);
    double f1 = this->at(index  ).Interp( cosangle );
    double f2 = this->at(index+1).Interp( cosangle );

  return Interp1( E1, f1, E2, f2, interpcode, elab );

}

/*! @brief Reconstruct normalized distribution at given incident energy
@param[in,out] result : normalized distribution
@param[in] eps        : accuracy of the interpolation
@note resut->C2() will be recognized as the incident energy 
*/
void LTT2::get_f_da( TAB1* result, const double eps ) const {

  Record a;

    this->Interp( a.E( result->C2() ) );  result->Set( a );

  return;

}


/*! @brief Calculate integral (tentative) */
double LTT2::I() const {
// integral (to be checled if this method is valid)

  double x1,x2,y1,y2, sum(0.);

  for( int i=0; i<this->NE()-1; i++ ) {

    x1 = this->E(i  );
    x2 = this->E(i+1);
    y1 = this->at(i  ).I();
    y2 = this->at(i+1).I();

    sum += Integral1( x1, y1, x2, y2, this->InterpCode(i)%20 );

  }

  return sum;

/*
  TAB1 temp;

  double sum = 0.0;

  for( int i=0; i<NE()-1; i++ ) {

    Integrate( ary[i], ary[i+1], this->InterpCode(i), temp.Clear(), 1000 );

    double a = temp.Integral();

    sum += temp.Integral( eps );

  }

  return sum;
*/

}



LTT2& LTT2::operator+=( const double val ) {

  for( int i=0; i<this->NE(); i++ ) this->at(i).operator+=( val );  return *this;

}

LTT2& LTT2::operator-=( const double val ) {

  for( int i=0; i<this->NE(); i++ ) this->at(i).operator-=( val );  return *this;

}

LTT2& LTT2::operator*=( const double val ) {

  for( int i=0; i<this->NE(); i++ ) this->at(i).operator*=( val );  return *this;

}

LTT2& LTT2::operator/=( const double val ) {

  for( int i=0; i<this->NE(); i++ ) this->at(i).operator/=( val );  return *this;

}





/*
void LTT2::Integral( TAB1* ans, const int nrandom ) const {
// under development
// integrate with Monte Carlo method to be used for e.g., angle-integral spectra
// nrandom : number of random numbers ( default = 1000 )

  for( int i=0; i<this->NE()-1; i++ ) {

    Integrate( this->at(i), this->at(i+1), this->InterpCode(i), ans, nrandom );

  }

  return;

}
*/

/*
void LTT2::Integral( TAB1* ans, const int nrandom ) const {
// integrate with Monte Carlo method to be used for e.g., angle-integral spectra
// nrandom : number of random numbers ( default = 1000 )

  TAB1 temp;

  for( int i=0; i<this->NE()-1; i++ ) {

    temp.Clear();

    Integrate( this->at(i), this->at(i+1), this->InterpCode(i), &temp, nrandom );

    *ans += temp;

  }

  return;

}
*/
