#if !defined(___Class_Nodes)
    #define  ___Class_Nodes

#include <iostream>
#include <utility>
#include <string>
#include "ReconResonance/RmatrixLimited/numeric/Approx.h"
#include "ReconResonance/RmatrixLimited/endf/TAB2.h"

using namespace std;
using namespace Numeric;

/* General functions for TAB2 + deque<Record> */

template<class T> void store( TAB2* o, deque<T>* array, istream& is ) {

  o->Store( is );

    array->resize( o->NZ() );

      for( int i=0; i<o->NZ(); i++ ) array->at(i).Store( is );

  return;

}

template<class T> void read( TAB2* o, deque<T>* array, const char* file_name ) {

  ifstream ifs;

    ifs.open( file_name, ios::in );

      if( ! ifs.is_open() ) {

        cerr << "!! Failed to open file : " << file_name << endl;  return;

      }

      store( o, array, ifs );

    ifs.close();

  return;

}

template<class T> void write( const TAB2* o, const deque<T>* array, ostream& os, const int MAT, const int MF, const int MT, int* NS ) {

  if( o->NZ() == 0 ) return;

  o->Write( os, MAT,MF,MT,NS );

    for( int i=0; i<o->NZ(); i++ ) array->at(i).Write( os, MAT,MF,MT,NS );

  return;

}

template<class T> void write( const TAB2* o, const deque<T>* array, const char* file_name ) {

  ofstream ofs( file_name );

    o->Write( ofs );

      for( int i=0; i<o->NZ(); i++ ) array->at(i).Write( ofs );

  return;

}

template<class T> unsigned long number_of_lines( const TAB2* o, const deque<T>* array ) {

  unsigned long n = o->NumberOfLines();

    for( int i=0; i<o->NZ(); i++ ) n += array->at(i).NumberOfLines();

  return n;

}

template<class T> void set_array( TAB2* o, deque<T>* array, const TAB2& a, const deque<T>& b ) {

  if( &a != o ) o->Set( a );
  if( &b != array ) *array = b;
  return;

}

template<class T> void set_array( TAB2* o, deque<T>* array, const int n, const int interpcode ) {

  o->Set( n, interpcode );  array->resize( n );  return;

}

template<class T> void clear( TAB2* o, deque<T>* array ) {

  o->Clear();  array->clear();  return;

}

template<class T> void push_front( TAB2* o, deque<T>* array, const T& r, const int interpcode ) {

  if( r.empty() ) return;

  if( ! array->empty() && array->front() == r ) return;

    o->PushFront( interpcode );  array->push_front( r );

  return;

}

template<class T> void push_back( TAB2* o, deque<T>* array, const T& r, const int interpcode ) {

  if( r.empty() ) return;

  if( ! array->empty() && array->back() == r ) return;

    o->PushBack( interpcode );  array->push_back( r );

  return;

}

template<class T> void push_front( TAB2* o, deque<T>* array, const TAB2& a, const deque<T>& b, const int interpcode ) {

  if( b.empty() ) return;

  if( array->empty() ) return set_array( o, array, a, b );

  if( array->front().C2() > b.back().C2() ) return;

  TAB2 aa;  deque<T> bb;

    set_array( &aa, &bb, a, b );

    aa.C1() = o->C1();  aa.C2() = o->C2();
    aa.L1() = o->L1();  aa.L2() = o->L2();

    push_back( &aa, &bb, *o, *array, interpcode );

  return set_array( o, array, aa, bb );

}

template<class T> void push_back( TAB2* o, deque<T>* array, const TAB2& a, const deque<T>& b, const int interpcode ) {

  if( b.empty() ) return;

  if( array->empty() ) return set_array( o, array, a,b );

  if( array->back().C2() > b.front().C2() ) return;

  if( eq( array->back().C2(), b.front().C2() ) && array->back() != b.front() ) {

    push_back( o, array, b.front(), a.Front().INT() );

  } else push_back( o, array, b.front(), interpcode );

  for( int i=1; i<a.NZ(); i++ ) push_back( o, array, b[i], a.InterpCode(i-1) );

  return;

}

template<class T> void pop_front( TAB2* o, deque<T>* array, const int num ) {

  for( int i=0; i<num; i++ ) {

    o->PopFront();  array->pop_front();

  }

  return;

}

template<class T> void pop_back( TAB2* o, deque<T>* array, const int num ) {

  for( int i=0; i<num; i++ ) {

    o->PopBack();  array->pop_back();

  }

  return;

}

template<class T> void erase( TAB2* o, deque<T>* array, const int index ) {

  typename deque<T>::iterator it = array->begin();  int i=0;

    for( it = array->begin(); it != array->end(); ) {

      if( i == index ) { it = array->erase(it);  break; } it++; i++;

    }

    o->Erase(index);

  return;

}

template<class T> void erase( TAB2* o, deque<T>* array, const double C2 ) {

  for( int i=0; i<array->size(); i++ )

    if( eq( array->at(i).C2(), C2 ) ) return erase( o, array, i );

  return;

}

template<class T> void thin( TAB2* o, deque<T>* array ) {

  int n = array->size();  vector<double> a;

    if( n <= 2 ) return;

    if( n%2 == 1 ) {

      for( int i=0; i<n; i++ ) if( i%2 == 1 ) a.push_back( array->at(i).C2() );

    }

    else {

      for( int i=1; i<n; i++ ) if( i%2 == 0 ) a.push_back( array->at(i).C2() );

    }

    for( int i=0; i<a.size(); i++ ) erase( o, array, a[i] );

  return;

}

template<class T> void cut_below( TAB2* o, deque<T>* array, const double C2 ) {

  if( array->empty() ) return;
  if( C2 <= array->front().C2() ) return;
  if( C2 >  array->back ().C2() ) return clear( o, array );

  int npop = 0;

  for( int i=o->NZ()-1; i>=0; i-- ) if( array->at(i).C2() <= C2 ) npop++;

  return pop_front( o, array, npop-1 );

}

template<class T> void cut_above( TAB2* o, deque<T>* array, const double C2 ) {

  if( array->empty() ) return;
  if( C2 >= array->back ().C2() ) return;
  if( C2 <  array->front().C2() ) return clear( o, array );

  int npop = 0;

  for( int i=0; i<o->NZ(); i++ ) if( array->at(i).C2() >= C2 ) npop++;

  return pop_back( o, array, npop-1 );

}

template<class T> void put_clone( TAB2* o, deque<T>* array, const double C2 ) {

  if( array->empty() ) return;

  if( C2 < array->front().C2() ) {

    T a = array->front();  a.C2(C2);  push_front( o, array, a, o->Front().INT() );

  }

  if( C2 > array->back().C2() ) {

    T a = array->back();  a.C2(C2);  push_back( o, array, a, o->Back().INT() );

  }

  return;

}

template<class T> void trim_lower( TAB2* o, deque<T>* array, const double C2 ) {

  if( array->empty() ) return;

  if( C2 < array->front().C2() ) {

    T a = array->front();  a.C2(C2);  push_front( o, array, a, o->Front().INT() );

  }

  return;

}

template<class T> void trim_upper( TAB2* o, deque<T>* array, const double C2 ) {

  if( array->empty() ) return;

  if( C2 > array->back().C2() ) {

    T a = array->back();  a.C2(C2);  push_back( o, array, a, o->Back().INT() );

  }

  return;

}


template<class T> bool find( const TAB2* o, const deque<T>* array, const double C2 ) {

  for( int i=0; i<o->NZ(); i++ ) if( eq( C2, array->at(i).C2() ) ) return true;

  return false;

}

template<class T> bool seek( const TAB2* o, const deque<T>* array, const double C2 ) {

  if( array->empty() ) return false;

  if( C2 < array->front().C2() ) return false;
  if( C2 > array->back ().C2() ) return false;

  return true;

}

template<class T> bool seek( const TAB2* o, const deque<T>* array, const double C2, int* index, int* interpcode ) {

  if( ! seek( o, array, C2 ) ) return false;

  for( int i=0; i<o->NZ()-1; i++ ) if( array->at(i).C2() <= C2 && C2 < array->at(i+1).C2() ) { *index = i;  break; }

  *interpcode = o->InterpCode( *index );

  return true;

}







// I/O stream
/*
template<class Record> istream& operator>>( istream& is, Nodes<Record>& ary ) {

  ary.Store( is );  return is;

}


template<class Record> ostream& operator<<( ostream& os, const Nodes<Record>& ary ) {

  ary.Write( os );  return os;

}
*/
//template<class Record> istream& operator>>( istream& is,       Nodes<Record>& ary );
//template<class Record> ostream& operator<<( ostream& os, const Nodes<Record>& ary );

#endif
