#if !defined(___Class_Parameter)
    #define  ___Class_Parameter

#include <iostream>
#include <iomanip>
#include <cmath>
////#include <TObject.h>

using namespace std;

namespace Numeric {

/*!
@class Parameter
@brief Container which stores a pair of "Value" and its "Error" (and "Name" if necessary)
*/

class Parameter {

  double value, error;  string text;

 public:

  Parameter();
  Parameter( const double val, const double err = 0., const string name = "" );
  Parameter( const Parameter& o );
  virtual ~Parameter();

  Parameter* Set( const double val, const double err = 0., const string name = "" );
  Parameter* Set( const Parameter& o );

  double  Value() const { return value; } /*! @brief Get value */
  double& Value()       { return value; }
  double  Error() const { return error; } /*! @brief Get uncertainty */
  double& Error()       { return error; }
  string  Name () const { return text;  } /*! @brief Get name of this object */
  string& Name ()       { return text;  }

  /*! @brief Get uncertainty of the value in percent */
  double ErrorPercent() const { return (value == 0.0) ? 0.0 : 100.*error/fabs(value); }

  /*! @brief Set value */
  Parameter* Value( const double val ) { value = val; return this; }

  /*! @brief Set uncertainty */
  Parameter* Error( const double err ) { error = err; return this; }

  /*! @brief Set uncertainty in percent */
  Parameter* ErrorPercent( const double percent ) { error = fabs(value)*percent/100.; return this; }

  /*! @brief Set name */
  Parameter* Name( const string name ) { text = name;  return this; }

  /*! @brief Set value of uncertainty to zero (to be used for parameter search) */
  Parameter* Fix() { return this->Error(0.0); }

  /*! @brief Check if value of uncertainty is larger than zero (to be used for parameter search) */
  bool is_free() const { return ( error > 0. ); } 

  void Clear();

  void Print     ( ostream& os = cout ) const;
  void PrintLaTex( ostream& os = cout ) const;

  Parameter& operator*=( const double val );
  Parameter& operator/=( const double val );

  Parameter& operator*=( const Parameter& o );
  Parameter& operator/=( const Parameter& o );

  bool operator==( const Parameter& o ) const;
  bool operator!=( const Parameter& o ) const;

  double Perturb( const double eps = 1.e-5 );

  //ClassDef( Numeric::Parameter, 1 );

};

ostream& operator<<( ostream& os, const Parameter& o );

}

#endif
